/*
 * A dataspace server exposing file contents accessed via the filesystem client
 * interface.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <ipc/thread.h>

#include <stdio.h>
#include <stdlib.h>

#include <mem/memory_incremental.h>
#include <fsserver/page_queue_shared.h>
#include <fsserver/pages.h>
#include <fsserver/resource_server.h>
#include <fsserver/client_file_opener.h>



/* Default number of pages for files. */

const unsigned int MEMORY_PAGES = 20;



/* Server program. */

int main(int argc, char *argv[])
{
  long err;

  /* Introduce concurrency control. */

  err = ipc_thread_init();

  if (err)
  {
    printf("Initialisation error: %s\n", l4sys_errtostr(err));
    return 1;
  }

  unsigned int memory_pages = MEMORY_PAGES;

  if (argc > 1)
    memory_pages = atoi(argv[1]);

  /* Some memory plus infrastructure. */

  MemoryIncremental mem(memory_pages);
  PageQueueShared queue;
  Pages pages(&mem, &queue);
  FileObjectRegistry registry(&pages);
  ClientFileOpener opener(&registry);

  /* Register a server associating it with the given object. */

  const char *server_name = (argc > 2) ? argv[2] : "server";

  ResourceServer server(&opener);
  err = server.bind(server_name);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server using %d pages...\n", memory_pages);
  server.start();
  return 0;
}
